type TopicEventHandler<T> = (payload: T) => unknown | Promise<unknown>;
type TopicSubscriptionType = "before" | "on" | "after";
type Transformer<T, K> = (payload: T) => K;
type PipeCheck<T> = (payload: T) => boolean;
interface TopicSubscription<T> {
    /** Call this function to remove this subscription */
    unsubscribe: () => void;
    /** The original handler function passed in for this subscription */
    handler: TopicEventHandler<T>;
    /** The type of subscription (one of "before" "on" or "after") */
    type: TopicSubscriptionType;
}
/**
 * A simple pub-sub like topic system for easy event publishing and listening
 */
declare class Topic<T> {
    private beforeListeners;
    private onListeners;
    private afterListeners;
    private removeSubscription;
    private createSubscription;
    /**
     * Register an event handler to run before normal handlers
     * @param handler A handler function which takes a payload and optional event name
     * @returns A subscription descriptor
     */
    before: (handler: TopicEventHandler<T>) => TopicSubscription<T>;
    /**
     * Register an event handler
     * @param handler A handler function which takes a payload and optional event name
     * @returns A subscription descriptor
     */
    subscribe: (handler: TopicEventHandler<T>) => TopicSubscription<T>;
    /**
     * Register an event handler to run after normal handlers
     * @param handler A handler function which takes a payload and optional event name
     * @returns A subscription descriptor
     */
    after: (handler: TopicEventHandler<T>) => TopicSubscription<T>;
    /**
     * Publish data and an optional event name ot the topic
     * @param payload Data to publish for this topic
     */
    publish: (payload: T) => void;
    /**
     * Automatically convert events from this topic and publish to another topic
     * @param topic The topic to pipe to
     * @param transformer A function to transform the payload before publishing
     */
    pipe: <K>(topic: Topic<K>, transformer: Transformer<T, K>) => void;
    /**
     * Automatically convert events from this topic and publish to another topic.
     * If the transformer returns void, the pipe will short-circuit instead.
     * @param topic The topic to pipe to
     * @param transformer A function to transform the payload before publishing or returns void to skip publishing
     */
    optionalPipe: <K>(topic: Topic<K>, check: PipeCheck<T>, transformer: Transformer<T, K>) => void;
}

declare const CAMPER_DEFAULT_AVATAR = "https://cdn.astral.camp/general-assets/default-avatar.png";
declare const CAMPER_DEFAULT_COLOR = "#FFFFFF";
declare const CAMPER_HANDLE_REGEX: RegExp;
declare const CAMPER_DISPLAY_NAME_MAX_LENGTH = 32;
declare const CAMPER_TAGLINE_MAX_LENGTH = 120;
declare const STREAMING_SERVICES: readonly ["twitch"];
declare const CHAT_SERVICES: readonly ["relay", "discord", "twitch"];
type StreamingService = (typeof STREAMING_SERVICES)[number];
type ChatService = (typeof CHAT_SERVICES)[number];
declare const RELAY_OBJECT_TYPES: readonly ["chat", "post", "concoction", "plant", "vibecheck", "camper", "character", "token", "letter", "starboard", "portent", "note", "shortlink"];
type RelayObject = (typeof RELAY_OBJECT_TYPES)[number];

type MediaAttachmentType = "image" | "video" | "audio" | "gifv" | "unknown";

type RelayID<T extends RelayObject> = `${T}_${string}`;

interface APICamperServiceConnections {
    mastodon?: {
        handle: string;
        displayName: string;
        id: string;
    };
    twitch?: {
        handle: string;
        displayName: string;
        color: string;
        id: string;
        creatorMode: boolean;
    };
    discord?: {
        handle: string;
        displayName: string;
        id: string;
    };
    prose?: {
        handle: string;
    };
}
/**
 * A small representation of a Camper
 */
interface APICamper {
    id: RelayID<"camper">;
    handle: string;
    name: string;
    pronouns: string | null;
    avatar: string;
    color: string;
    services: APICamperServiceConnections;
}
interface APIMediaAttachment {
    type: MediaAttachmentType;
    url: string;
    description: string | null;
    warning: string | null;
    credit: string | null;
    mimeType: string | null;
}

interface APIRelayMessageMetadata {
    twitch: {
        subscriber: boolean;
        vip: boolean;
    } | null;
    discord: {
        messageId: string;
        channelId: string;
        parentChannelId: string | null;
        isThread: boolean;
    } | null;
}
interface APIRelayMessage {
    id: RelayID<"chat">;
    camper: APICamper | null;
    content: string;
    richContent: string;
    author: string;
    avatar: string;
    color: string;
    source: ChatService;
    sourceChannel: string;
    sourceId: string;
    replyTo: RelayID<"chat"> | null;
    createdAt: string;
    attachments: APIMediaAttachment[];
    metadata: APIRelayMessageMetadata;
}

declare const POST_SERVICES: readonly ["relay", "mastodon", "prose", "vibe-check"];

type PostService = (typeof POST_SERVICES)[number];

interface APIRelayPost {
    id: RelayID<"post">;
    camper: APICamper | null;
    source: PostService;
    sourceChannel: string;
    sourceId: string;
    sourceUrl: string;
    title: string | null;
    summary: string | null;
    author: string;
    published: string;
    content: string;
    tags: string[];
    attachments: APIMediaAttachment[];
    createdAt: string;
    updatedAt: string | null;
}

interface BaseAPIStreamEvent {
    /** The service this came from */
    source: StreamingService;
    /** The channel ID of the streamer (ID, not handle) */
    sourceChannel: string;
    /** The Camper who is broadcasting */
    broadcastingCamper: APICamper;
}
interface APIStreamLiveEvent extends BaseAPIStreamEvent {
    /**
     * An ISO 8601 timestamp of when the stream went live
     */
    timestamp: string;
}
interface APIStreamRequestEvent extends BaseAPIStreamEvent {
    /** The title of the request */
    request: string;
    /**
     * The message associated with the request
     * If no message is supplied, this will be an empty string
     */
    message: string;
    /**
     * The computed name of the user who made the request.
     * If this was from a known Camper, this will be their preferred Camp name
     * Otherwise, it will be derived from the upstream event
     */
    username: string;
    /**
     * The computed avatar of the user who made the request.
     * If there was no avatar to use, this will be null
     */
    avatar: string | null;
    /**
     * The computed color associated with the user who made the request.
     * If this is a camper, this will be their favorite color setting
     * Othherwise, it will be derived from the upstream event
     */
    color: string;
    /** If set, represents the Camper who made the request */
    camper: APICamper | null;
}
interface APIStreamFollowEvent extends BaseAPIStreamEvent {
    /**
     * The computed name of the user who followed the broadcaster.
     * If this was from a known Camper, this will be their preferred Camp name
     * Otherwise, it will be derived from the upstream event
     */
    username: string;
    /**
     * The computed avatar of the user who followed the broadcaster.
     * If there was no avatar to use, this will be null
     */
    avatar: string | null;
    /**
     * The computed color associated with the user who followed.
     * If this is a camper, this will be their favorite color setting
     * Othherwise, it will be derived from the upstream event
     */
    color: string;
    /** If set, represents the Camper who followed the broadcaster */
    camper: APICamper | null;
}
interface APIStreamSubEvent extends BaseAPIStreamEvent {
    /**
     * The computed name of the user who subscribed to the broadcaster.
     * If this was from a known Camper, this will be their preferred Camp name
     * Otherwise, it will be derived from the upstream event
     */
    username: string;
    /**
     * The computed avatar of the user who subscribed.
     * If there was no avatar to use, this will be null
     */
    avatar: string | null;
    /**
     * The computed color associated with the user who subscribed.
     * If this is a camper, this will be their favorite color setting
     * Othherwise, it will be derived from the upstream event
     */
    color: string;
    /**
     * The message associated with the subcription,
     * or null if no message was provided
     */
    message: string | null;
    /** The duration of the subscription in months */
    months: number;
    /** If set, represents the Camper who subscribed to the broadcaster */
    camper: APICamper | null;
}
interface APIStreamGiftedSubEvent extends BaseAPIStreamEvent {
    /**
     * The computed name of the user who gifted a subscription.
     * If this was from a known Camper, this will be their preferred Camp name
     * Otherwise, it will be derived from the upstream event
     * If the gift was anonymous, this will be a placeholder name
     * such as "A Generous Soul"
     */
    gifterName: string;
    /** If set, represents the Camper who gifted the subscription */
    gifterCamper: APICamper | null;
    /**
     * The computed avatar of the user who gifted the subscription.
     * If there was no avatar to use, this will be null
     */
    gifterAvatar: string | null;
    /**
     * The computed color associated with the user who gifted the subscription.
     * If this is a camper, this will be their favorite color setting
     * Othherwise, it will be derived from the upstream event
     */
    gifterColor: string;
    /**
     * The computed name of the user who received the gifted subscription.
     * If this was from a known Camper, this will be their preferred Camp name
     * Otherwise, it will be derived from the upstream event
     */
    recipientName: string;
    /** If set, represents the Camper who received the gift */
    recipientCamper: APICamper | null;
    /**
     * The computed avatar of the user who received the gifted subscription.
     * If there was no avatar to use, this will be null
     */
    recipientAvatar: string | null;
    /**
     * The computed color associated with the user who received the gifted subscription.
     * If this is a camper, this will be their favorite color setting
     * Othherwise, it will be derived from the upstream event
     */
    recipientColor: string;
}
interface APIStreamMassGiftedSubEvent extends BaseAPIStreamEvent {
    /** The number of subscriptions gifted */
    count: number;
    /**
     * The computed name of the user who gifted a subscription.
     * If this was from a known Camper, this will be their preferred Camp name
     * Otherwise, it will be derived from the upstream event
     * If the gift was anonymous, this will be a placeholder name
     * such as "A Generous Soul"
     */
    username: string;
    /**
     * The computed avatar of the user who gifted the subscriptions.
     * If there was no avatar to use, this will be null
     */
    avatar: string | null;
    /**
     * The computed color associated with the user who gifted the subscriptions.
     * If this is a camper, this will be their favorite color setting
     * Othherwise, it will be derived from the upstream event
     */
    color: string;
    /** If set, represents the Camper who gifted the subscriptions */
    camper: APICamper | null;
}
interface APIStreamRaidEvent extends BaseAPIStreamEvent {
    /**
     * The computed name of the user who raided the broadcaster.
     * If this was from a known Camper, this will be their preferred Camp name
     * Otherwise, it will be derived from the upstream event
     */
    username: string;
    /**
     * The computed avatar of the user who raided.
     * If there was no avatar to use, this will be null
     */
    avatar: string | null;
    /**
     * The computed color associated with the user who raided.
     * If this is a camper, this will be their favorite color setting
     * Othherwise, it will be derived from the upstream event
     */
    color: string;
    /** The number of viewers the raider brought along */
    count: number;
    /** If set, represents the Camper who raided the broadcaster */
    camper: APICamper | null;
}
interface APIStreamCheerEvent extends BaseAPIStreamEvent {
    /**
     * The computed name of the user who cheered the broadcaster.
     * If this was from a known Camper, this will be their preferred Camp name
     * Otherwise, it will be derived from the upstream event
     */
    username: string;
    /**
     * The computed avatar of the user who cheered.
     * If there was no avatar to use, this will be null
     */
    avatar: string | null;
    /**
     * The computed color associated with the user who cheered.
     * If this is a camper, this will be their favorite color setting
     * Othherwise, it will be derived from the upstream event
     */
    color: string;
    /** The amount of currency cheered with (Bits for Twitch) */
    amount: number;
    /** If set, represents the Camper who cheered the broadcaster */
    camper: APICamper | null;
}

type APISocketEvent = {
    event: "chat:message:created";
    data: APIRelayMessage;
} | {
    event: "post:created";
    data: APIRelayPost;
} | {
    event: "stream:live";
    data: APIStreamLiveEvent;
} | {
    event: "stream:request";
    data: APIStreamRequestEvent;
} | {
    event: "stream:follow";
    data: APIStreamFollowEvent;
} | {
    event: "stream:subscription";
    data: APIStreamSubEvent;
} | {
    event: "stream:gifted-subscription";
    data: APIStreamGiftedSubEvent;
} | {
    event: "stream:mass-gifted-subscription";
    data: APIStreamMassGiftedSubEvent;
} | {
    event: "stream:raid";
    data: APIStreamRaidEvent;
} | {
    event: "stream:cheer";
    data: APIStreamCheerEvent;
};

interface APIMailLetter {
    id: RelayID<"letter">;
    from: APICamper;
    content: string;
    createdAt: string;
    read: boolean;
}

declare const NOTE_VISIBILITY_OPTIONS: readonly ["public", "private"];
declare const NOTE_CATEGORY_OPTIONS: readonly ["game", "game:botany", "game:chemistry", "game:economy", "game:badge", "game:item", "game:portent", "social", "social:mastodon", "social:prose", "social:twitch", "social:discord", "camp", "camp:mail", "camp:vibe-check", "camp:shortlink", "camp:camper"];

type NoteVisibility = (typeof NOTE_VISIBILITY_OPTIONS)[number];
type NoteCategory = (typeof NOTE_CATEGORY_OPTIONS)[number];

interface APINote {
    id: RelayID<"note">;
    visibility: NoteVisibility;
    content: string;
    category: NoteCategory;
    createdAt: string;
}

/**
 * Represents a Camper's profile and character information
 * Does not include settings or private information
 */
interface APICamperProfile {
    camperId: RelayID<"camper">;
    handle: string;
    name: string;
    pronouns: string;
    avatar: string;
    color: string;
    tagline: string;
    website: string;
    services: APICamperServiceConnections;
}

interface APIShortlink {
    alias: string;
    href: string;
}

interface RelayClientConfig {
    socketHost: string;
    socketPort: number;
    socketProtocol: "ws" | "wss";
    socketPath: string;
    socketQuery: string;
    httpHost: string;
    httpPort: number;
    httpProtocol: "http" | "https";
}
declare class RelayClient {
    token: string;
    private url;
    private socket;
    private apiURL;
    connected: Topic<void>;
    disconnected: Topic<void>;
    error: Topic<Error>;
    Events: {
        Chat: {
            new: Topic<APIRelayMessage>;
        };
        Post: {
            new: Topic<APIRelayPost>;
        };
        Stream: {
            live: Topic<APIStreamLiveEvent>;
            request: Topic<APIStreamRequestEvent>;
            follow: Topic<APIStreamFollowEvent>;
            subscription: Topic<APIStreamSubEvent>;
            giftedSubscription: Topic<APIStreamGiftedSubEvent>;
            massGiftedSubscription: Topic<APIStreamMassGiftedSubEvent>;
            raid: Topic<APIStreamRaidEvent>;
            cheer: Topic<APIStreamCheerEvent>;
        };
    };
    constructor(token: string, configOverrides?: Partial<RelayClientConfig>);
    /**
     * Construct a qualified API route
     *
     * @param route - The route to construct, includng the leading slash
     * @return The qualified API route
     *
     * @example
     * const resourceRoute = this.getAPIRoute("/resource/list");
     */
    private getAPIRoute;
    get isConnected(): boolean;
    /**
     * Close the socket connection to the Stardust Relay
     */
    close(): void;
    /**
     * Get an array of posts made by a camper
     *
     * @param camperId - The ID of the camper to fetch posts for
     * @param source (optional) - The source to fetch posts from
     * @return A promise which resolves to an array of APIRelayPost objects
     */
    getCamperPosts(camperId: string, source?: PostService): Promise<APIRelayPost[]>;
    /**
     * Get an array of posts from a specific source
     *
     * @param source - The source to fetch posts from
     * @return A promise which resolves to an array of APIRelayPost objects
     */
    getPosts(source: PostService): Promise<APIRelayPost[]>;
    /**
     * Get a single post by its ID
     *
     * @param postId - The ID of the post to fetch
     * @return A promise which resolves to an APIRelayPost object
     */
    getPost(postId: string): Promise<APIRelayPost>;
    /**
     * Fetch up to the most recent 100 messages from a chat log
     *
     * @param service - The chat service to fetch from
     * @param channel - The channel to fetch from
     *
     * @return A promise which resolves to an array of APIRelayMessage objects
     */
    getChatLog(service: ChatService, channel: string): Promise<APIRelayMessage[]>;
    /**
     * Fetch an array of all campers as APICamper objects
     * @return A promise which resolves to an array of APICamper objects
     */
    getDirectory(): Promise<APICamper[]>;
    /**
     * Fetch a Camper by their handle
     * This is a smaller representation of a Camper
     *
     * @param handle - The handle of the Camper to fetch
     * @return A promise which resolves to an APICamper object
     */
    getCamper(handle: string): Promise<APICamper>;
    /**
     * Fetch a Camper's profile by their handle
     * This contains extended information that is not available from getCamper()
     *
     * @param handle - The handle of the Camper to fetch
     * @return A promise which resolves to an APICamperProfile object
     */
    getProfile(handle: string): Promise<APICamperProfile>;
    /**
     * Fetch the profile of the currently authenticated camper
     *
     * @return A promise which resolves to an APICamperProfile object
     */
    getMyProfile(): Promise<APICamperProfile>;
    /**
     * Fetch the most recent public note history for a specific camper
     * or the whole community
     *
     * @param handle (optional) - The handle of the camper to fetch notes for if set
     * @return A promise which resolves to an array of APINote objects
     */
    getPublicNotes(handle?: string): Promise<APINote[]>;
    /**
     * Fetch the most recent private note history for the currently authenticated camper
     *
     * @return A promise which resolves to an array of APINote objects
     */
    getMyNotes(): Promise<APINote[]>;
    /**
     * Fetch the mailbox of the currently authenticated camper
     *
     * @return A promise which resolves to an array of APIMailLetter objects
     */
    getMailbox(): Promise<APIMailLetter[]>;
    /**
     * Fetch the list of shortlinks created by the currently authenticated camper
     * @return A promise which resolves to an array of APIShortlink objects
     */
    getShortlinks(): Promise<APIShortlink[]>;
    /**
     * Create a shortlink with the given alias and href
     *
     * @param href - The URL to link to
     * @param alias (optional) - The alias to use for the shortlink
     * @return A promise which resolves to the created shortlink
     */
    createShortlink(href: string, alias?: string): Promise<string>;
    /**
     * Delete a shortlink by its alias
     *
     * @param alias - The alias of the shortlink to delete
     * @return A promise which resolves when the shortlink is deleted
     */
    deleteShortlink(alias: string): Promise<string>;
    twitchBotSay(message: string): Promise<void>;
}

export { CAMPER_DEFAULT_AVATAR, CAMPER_DEFAULT_COLOR, CAMPER_DISPLAY_NAME_MAX_LENGTH, CAMPER_HANDLE_REGEX, CAMPER_TAGLINE_MAX_LENGTH, RelayClient as default };
export type { APICamper, APICamperProfile, APIRelayMessage, APIRelayPost, APISocketEvent, APIStreamCheerEvent, APIStreamFollowEvent, APIStreamGiftedSubEvent, APIStreamLiveEvent, APIStreamMassGiftedSubEvent, APIStreamRaidEvent, APIStreamRequestEvent, APIStreamSubEvent, ChatService };
