// src/errors.ts
class SocketError extends Error {
  name = "SocketError";
  originalError;
  constructor(message, originalError) {
    super(message);
    this.originalError = originalError;
  }
}

// src/messages.ts
function wrap(data) {
  return ["data", data];
}
function deserialize(data) {
  try {
    const parsedData = JSON.parse(data);
    if (typeof parsedData === "number") {
      return ["heartbeat", parsedData];
    }
    if (parsedData !== null && Array.isArray(parsedData) && parsedData.length === 2 && parsedData[0] === "data") {
      const data2 = parsedData[1];
      return ["data", data2];
    }
    throw new SocketError("Failed to parse internal message shape");
  } catch (error) {
    throw new SocketError("Failed to parse incoming message shape: Failed to deserialize");
  }
}

// src/socket.ts
var ONE_SECOND = 1000;
var HEARTBEAT_INTERVAL = 30 * ONE_SECOND;
var MAX_RECONNECT_DELAY = 30 * ONE_SECOND;

class ManagedSocket {
  url;
  socket;
  onMessage = () => {};
  onError = () => {};
  onConnected = () => {};
  onDisconnected = () => {};
  _reconnectAttempts = 0;
  _messageQueue = [];
  _debugMode = false;
  _offline = false;
  constructor(url, handlers = {}) {
    this.url = url;
    this.onMessage = handlers.onMessage ?? this.onMessage;
    this.onError = handlers.onError ?? this.onError;
    this.onConnected = handlers.onConnected ?? this.onConnected;
    this.onDisconnected = handlers.onDisconnected ?? this.onDisconnected;
    this.socket = this.connect();
    setInterval(() => {
      this.heartbeat();
    }, HEARTBEAT_INTERVAL);
    window.addEventListener("offline", () => {
      this._offline = true;
      if (this.isConnected) {
        this.debugMessage("Internet connection lost, closing socket...");
        this.close();
      }
    });
    window.addEventListener("online", () => {
      this._offline = false;
      if (!this.isConnected) {
        this.debugMessage("Internet connection restored, reconnecting socket...");
        this.attemptReconnection();
      }
    });
  }
  heartbeat() {
    if (this.socket?.readyState === WebSocket.OPEN) {
      this.socket.send(JSON.stringify(Date.now()));
    }
  }
  debugMessage(...args) {
    if (this._debugMode) {
      const output = ["[Socket]", ...args];
      console.log(...output);
    }
  }
  sendQueuedMessages() {
    const messagesToSend = [...this._messageQueue];
    this._messageQueue = [];
    for (const message of messagesToSend) {
      this.send(message);
    }
  }
  attemptReconnection() {
    if (this._offline) {
      this.debugMessage("Browser offline; skipping reconnection attempt");
      return;
    }
    const delay = Math.min(ONE_SECOND * 2 ** this._reconnectAttempts, MAX_RECONNECT_DELAY);
    this.debugMessage(`Attempting to reconnect in ${delay}ms`);
    setTimeout(() => {
      this._reconnectAttempts++;
      this.connect();
    }, delay);
  }
  connect() {
    const newSocket = new WebSocket(this.url);
    newSocket.addEventListener("open", () => {
      this._reconnectAttempts = 0;
      this.onConnected();
      this.sendQueuedMessages();
    });
    newSocket.addEventListener("close", ({ wasClean, code, reason }) => {
      this.debugMessage("WebSocket closed", {
        wasClean,
        code,
        reason
      });
      this.onDisconnected({
        wasClean,
        code,
        reason
      });
      if (!this._offline) {
        this.debugMessage("Attempting automatic reconnection");
        return this.attemptReconnection();
      }
    });
    newSocket.addEventListener("error", (error) => {
      this.debugMessage("WebSocket error", error);
      const socketError = new SocketError("Socket error", new Error(error.currentTarget?.toString()));
      this.onError(socketError);
    });
    newSocket.addEventListener("message", (messageEvent) => {
      const [type, data] = deserialize(messageEvent.data);
      if (type === "heartbeat") {
        return;
      }
      this.debugMessage("Received message", data);
      this.onMessage(data);
    });
    this.socket = newSocket;
    return newSocket;
  }
  get isConnected() {
    return this.socket?.readyState === WebSocket.OPEN;
  }
  debug(debugModeEnabled) {
    this._debugMode = debugModeEnabled;
  }
  close() {
    if (this.socket?.readyState !== WebSocket.OPEN) {
      throw new SocketError("Cannot close a socket that is not open");
    }
    this.socket.close();
  }
  send(data) {
    if (!this.socket || this.socket.readyState !== WebSocket.OPEN) {
      this.debugMessage("WebSocket is not open. Queuing message.");
      this._messageQueue.push(data);
      return;
    }
    this.socket.send(JSON.stringify(wrap(data)));
  }
}

/**
 * A simple pub-sub like topic system for easy event publishing and listening
 */
class Topic {
    beforeListeners = [];
    onListeners = [];
    afterListeners = [];
    removeSubscription = (handler, type) => {
        const isNotBeingRemoved = (subscription) => subscription.handler !== handler;
        switch (type) {
            case "before":
                this.beforeListeners = this.beforeListeners.filter(isNotBeingRemoved);
                break;
            case "on":
                this.onListeners = this.onListeners.filter(isNotBeingRemoved);
                break;
            case "after":
                this.afterListeners = this.afterListeners.filter(isNotBeingRemoved);
                break;
        }
    };
    createSubscription = (handler, type) => {
        return {
            handler,
            type,
            unsubscribe: () => {
                this.removeSubscription(handler, type);
            },
        };
    };
    /**
     * Register an event handler to run before normal handlers
     * @param handler A handler function which takes a payload and optional event name
     * @returns A subscription descriptor
     */
    before = (handler) => {
        const subscription = this.createSubscription(handler, "before");
        this.beforeListeners.push(subscription);
        return subscription;
    };
    /**
     * Register an event handler
     * @param handler A handler function which takes a payload and optional event name
     * @returns A subscription descriptor
     */
    subscribe = (handler) => {
        const subscription = this.createSubscription(handler, "on");
        this.onListeners.push(subscription);
        return subscription;
    };
    /**
     * Register an event handler to run after normal handlers
     * @param handler A handler function which takes a payload and optional event name
     * @returns A subscription descriptor
     */
    after = (handler) => {
        const subscription = this.createSubscription(handler, "after");
        this.afterListeners.push(subscription);
        return subscription;
    };
    /**
     * Publish data and an optional event name ot the topic
     * @param payload Data to publish for this topic
     */
    publish = (payload) => {
        for (const { handler } of this.beforeListeners) {
            handler(payload);
        }
        for (const { handler } of this.onListeners) {
            handler(payload);
        }
        for (const { handler } of this.afterListeners) {
            handler(payload);
        }
    };
    /**
     * Automatically convert events from this topic and publish to another topic
     * @param topic The topic to pipe to
     * @param transformer A function to transform the payload before publishing
     */
    pipe = (topic, transformer) => {
        this.subscribe((payload) => {
            topic.publish(transformer(payload));
        });
    };
    /**
     * Automatically convert events from this topic and publish to another topic.
     * If the transformer returns void, the pipe will short-circuit instead.
     * @param topic The topic to pipe to
     * @param transformer A function to transform the payload before publishing or returns void to skip publishing
     */
    optionalPipe = (topic, check, transformer) => {
        this.subscribe((payload) => {
            if (check(payload)) {
                topic.publish(transformer(payload));
            }
        });
    };
}

const CAMPER_DEFAULT_AVATAR = "https://cdn.astral.camp/general-assets/default-avatar.png";
const CAMPER_DEFAULT_COLOR = "#FFFFFF";
const CAMPER_HANDLE_REGEX = /^[a-zA-Z0-9_-]{3,32}$/;
const CAMPER_DISPLAY_NAME_MAX_LENGTH = 32;
const CAMPER_TAGLINE_MAX_LENGTH = 120;
var CamperAccessLevel;
(function (CamperAccessLevel) {
    CamperAccessLevel[CamperAccessLevel["Banned"] = -1] = "Banned";
    CamperAccessLevel[CamperAccessLevel["Normal"] = 0] = "Normal";
    CamperAccessLevel[CamperAccessLevel["Firewatch"] = 1] = "Firewatch";
    CamperAccessLevel[CamperAccessLevel["Moderator"] = 2] = "Moderator";
    CamperAccessLevel[CamperAccessLevel["Admin"] = 3] = "Admin";
})(CamperAccessLevel || (CamperAccessLevel = {}));

class RelayAPIError extends Error {
    status;
    statusText;
    constructor(status, statusText) {
        super(`[Relay API Error] ${status} ${statusText}`);
        this.status = status;
        this.statusText = statusText;
    }
}
async function apiGet(url, token) {
    const res = await fetch(url, {
        method: "GET",
        headers: {
            Authorization: `Bearer ${token}`,
        },
    });
    if (res.ok) {
        return res.json();
    }
    throw new RelayAPIError(res.status, res.statusText);
}
async function apiDelete(url, token) {
    const res = await fetch(url, {
        method: "DELETE",
        headers: {
            Authorization: `Bearer ${token}`,
        },
    });
    if (res.ok) {
        return res.json();
    }
    throw new RelayAPIError(res.status, res.statusText);
}
async function apiPost(url, token, body) {
    const res = await fetch(url, {
        method: "POST",
        headers: {
            Authorization: `Bearer ${token}`,
            "Content-Type": "application/json",
        },
        body: JSON.stringify(body),
    });
    if (res.ok) {
        return res.json();
    }
    throw new RelayAPIError(res.status, res.statusText);
}
const DEFAULT_CONFIG = {
    socketHost: "relay-ws.astral.camp",
    socketPort: 443,
    socketProtocol: "wss",
    socketPath: "/ws",
    socketQuery: "",
    httpHost: "relay.astral.camp",
    httpPort: 443,
    httpProtocol: "https",
};
class RelayClient {
    token;
    url = "https://relay-ws.astral.camp/ws";
    socket;
    apiURL;
    connected = new Topic();
    disconnected = new Topic();
    error = new Topic();
    Events = {
        Chat: {
            new: new Topic(),
        },
        Post: {
            new: new Topic(),
        },
        Stream: {
            live: new Topic(),
            request: new Topic(),
            follow: new Topic(),
            subscription: new Topic(),
            giftedSubscription: new Topic(),
            massGiftedSubscription: new Topic(),
            raid: new Topic(),
            cheer: new Topic(),
        },
    };
    constructor(token, configOverrides) {
        const { socketHost, socketPort, socketProtocol, socketPath, socketQuery, httpHost, httpPort, httpProtocol, } = Object.assign({}, DEFAULT_CONFIG, configOverrides);
        this.token = token;
        const socketConnURL = new URL(`${socketProtocol}://${socketHost}:${socketPort}${socketPath}`);
        socketConnURL.search = socketQuery;
        socketConnURL.searchParams.set("token", token);
        this.url = socketConnURL.toString();
        this.apiURL = new URL(`${httpProtocol}://${httpHost}:${httpPort}`).toString();
        this.socket = new ManagedSocket(this.url, {
            onConnected: () => {
                console.info("Stardust Relay connected");
                this.connected.publish();
            },
            onDisconnected: () => {
                console.info("Stardust Relay disconnected");
                this.disconnected.publish();
            },
            onError: (error) => {
                console.error(error);
                this.error.publish(error);
            },
            onMessage: (raw) => {
                const message = raw;
                switch (message.event) {
                    case "chat:message:created":
                        return this.Events.Chat.new.publish(message.data);
                    case "post:created":
                        return this.Events.Post.new.publish(message.data);
                    case "stream:live":
                        return this.Events.Stream.live.publish(message.data);
                    case "stream:request":
                        return this.Events.Stream.request.publish(message.data);
                    case "stream:follow":
                        return this.Events.Stream.follow.publish(message.data);
                    case "stream:subscription":
                        return this.Events.Stream.subscription.publish(message.data);
                    case "stream:gifted-subscription":
                        return this.Events.Stream.giftedSubscription.publish(message.data);
                    case "stream:mass-gifted-subscription":
                        return this.Events.Stream.massGiftedSubscription.publish(message.data);
                    case "stream:raid":
                        return this.Events.Stream.raid.publish(message.data);
                    case "stream:cheer":
                        return this.Events.Stream.cheer.publish(message.data);
                }
            },
        });
    }
    /**
     * Construct a qualified API route
     *
     * @param route - The route to construct, includng the leading slash
     * @return The qualified API route
     *
     * @example
     * const resourceRoute = this.getAPIRoute("/resource/list");
     */
    getAPIRoute(route) {
        return new URL(route, this.apiURL).toString();
    }
    get isConnected() {
        return this.socket.isConnected;
    }
    /**
     * Close the socket connection to the Stardust Relay
     */
    close() {
        this.socket.close();
    }
    // == API RESOURCES ==
    /**
     * Get an array of posts made by a camper
     *
     * @param camperId - The ID of the camper to fetch posts for
     * @param source (optional) - The source to fetch posts from
     * @return A promise which resolves to an array of APIRelayPost objects
     */
    async getCamperPosts(camperId, source) {
        if (source) {
            return apiGet(`/post/collection/${camperId}/${source}`, this.token);
        }
        return apiGet(`/post/collection/camper/${camperId}`, this.token);
    }
    /**
     * Get an array of posts from a specific source
     *
     * @param source - The source to fetch posts from
     * @return A promise which resolves to an array of APIRelayPost objects
     */
    async getPosts(source) {
        return apiGet(this.getAPIRoute(`/post/collection/source/${source}`), this.token);
    }
    /**
     * Get a single post by its ID
     *
     * @param postId - The ID of the post to fetch
     * @return A promise which resolves to an APIRelayPost object
     */
    async getPost(postId) {
        return apiGet(this.getAPIRoute(`/post/${postId}`), this.token);
    }
    /**
     * Fetch up to the most recent 100 messages from a chat log
     *
     * @param service - The chat service to fetch from
     * @param channel - The channel to fetch from
     *
     * @return A promise which resolves to an array of APIRelayMessage objects
     */
    async getChatLog(service, channel) {
        return apiGet(this.getAPIRoute(`/chat/log/${service}/${channel}`), this.token);
    }
    /**
     * Fetch an array of all campers as APICamper objects
     * @return A promise which resolves to an array of APICamper objects
     */
    async getDirectory() {
        return await apiGet("/profile/directory", this.token);
    }
    /**
     * Fetch a Camper by their handle
     * This is a smaller representation of a Camper
     *
     * @param handle - The handle of the Camper to fetch
     * @return A promise which resolves to an APICamper object
     */
    async getCamper(handle) {
        return await apiGet(`/profile/${handle}/summary`, this.token);
    }
    /**
     * Fetch a Camper's profile by their handle
     * This contains extended information that is not available from getCamper()
     *
     * @param handle - The handle of the Camper to fetch
     * @return A promise which resolves to an APICamperProfile object
     */
    async getProfile(handle) {
        return await apiGet(this.getAPIRoute(`/profile/${handle}`), this.token);
    }
    /**
     * Fetch the profile of the currently authenticated camper
     *
     * @return A promise which resolves to an APICamperProfile object
     */
    async getMyProfile() {
        return await apiGet(this.getAPIRoute("/profile"), this.token);
    }
    /**
     * Fetch the most recent public note history for a specific camper
     * or the whole community
     *
     * @param handle (optional) - The handle of the camper to fetch notes for if set
     * @return A promise which resolves to an array of APINote objects
     */
    async getPublicNotes(handle) {
        if (handle) {
            return apiGet(`/note/public/${handle}`, this.token);
        }
        return apiGet("/note/public", this.token);
    }
    /**
     * Fetch the most recent private note history for the currently authenticated camper
     *
     * @return A promise which resolves to an array of APINote objects
     */
    async getMyNotes() {
        return apiGet(this.getAPIRoute("/note"), this.token);
    }
    /**
     * Fetch the mailbox of the currently authenticated camper
     *
     * @return A promise which resolves to an array of APIMailLetter objects
     */
    async getMailbox() {
        return apiGet(this.getAPIRoute("/mail"), this.token);
    }
    /**
     * Fetch the list of shortlinks created by the currently authenticated camper
     * @return A promise which resolves to an array of APIShortlink objects
     */
    async getShortlinks() {
        return apiGet(this.getAPIRoute("/shortlink"), this.token);
    }
    // == API METHODS ==
    /**
     * Create a shortlink with the given alias and href
     *
     * @param href - The URL to link to
     * @param alias (optional) - The alias to use for the shortlink
     * @return A promise which resolves to the created shortlink
     */
    async createShortlink(href, alias) {
        return apiPost(this.getAPIRoute("/shortlink"), this.token, { href, alias });
    }
    /**
     * Delete a shortlink by its alias
     *
     * @param alias - The alias of the shortlink to delete
     * @return A promise which resolves when the shortlink is deleted
     */
    async deleteShortlink(alias) {
        return apiDelete(`/shortlink/${alias}`, this.token);
    }
    async twitchBotSay(message) {
        return apiPost(this.getAPIRoute("/twitch/bot/say"), this.token, { message });
    }
}

export { CAMPER_DEFAULT_AVATAR, CAMPER_DEFAULT_COLOR, CAMPER_DISPLAY_NAME_MAX_LENGTH, CAMPER_HANDLE_REGEX, CAMPER_TAGLINE_MAX_LENGTH, RelayClient as default };
